local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")

Sackville.Bag = {}
Sackville.Bag.__index = Sackville.Bag

local _

function Sackville.Bag:New(bagName, bagConfig, bagState)
    Sackville:Debug(1, "bag", "Adding new virtual bag: %s", bagName)
    local self = {}
    setmetatable(self, Sackville.Bag)

    local bagDefaults = Sackville.Options.bag
    self.name = bagName
    self.displayOrder = bagConfig.displayOrder
    self.bagType = bagConfig.bagType
    self.defaultPriority = bagConfig.defaultPriority or 1
    self.defaultSortCriteria = bagConfig.defaultSortCriteria
    self.minimized = bagState.minimized

    self.frame = self:CreateBagFrame(self.name, self.bagType, false)
    self.min_width = self.frame:GetWidth()
    self.max_width = bagDefaults.max_width  
    self.sections = {}
    return self
end

function Sackville.Bag:AddSection(sectionName, sectionOptions, sectionState)
    local bagDefaults = Sackville.Options.bag
    local section = Sackville.Section:New(self, sectionOptions, sectionState)
    section:SetMaxWidth(self.max_width - bagDefaults.margin_left 
        - bagDefaults.margin_right)
    self.sections[section:GetName()] = section
    return section
end

function Sackville.Bag:RemoveSection(sectionName)
    self.sections[sectionName] = nil
end

function Sackville.Bag:CreateBagFrame(bagName, bagType, moveable)
    Sackville:Debug(1, "bag", "CreateBagFrame: %s", bagName)
    local frameName = "SackvilleVB_" .. bagName
    local f = _G[frameName] or CreateFrame('Frame', frameName, UIParent)
    local bagDefaults = Sackville.Options.bag
    f:Hide()

    if bagType == PLAYER then
        f:SetBackdrop(bagDefaults.backdrop)
    elseif bagType == L["Bank"] then
        f:SetBackdrop(bagDefaults.backdrop_bank)
    end
    f:SetHeight(bagDefaults.min_height)

    f:SetFrameStrata(bagDefaults.strata)
    
    if (moveable) then
        f:SetMovable(true)
        f:EnableMouse(true)
        f:RegisterForDrag('LeftButton')
        f:SetScript('OnMouseDown', function(f, button)
         if button ~= 'LeftButton' then return end
         f.pOrigin = f.pOrigin or {}
         f.pOrigin.point, f.pOrigin.anchor, f.pOrigin.relPoint, f.pOrigin.x, f.pOrigin.y
            = f:GetPoint(1)
         f:StartMoving()
         f.pStart = f.pStart or {}
         f.pStart.x, f.pStart.y = select(4, f:GetPoint(1))
        end)
        f:SetScript('OnMouseUp', function(f, button)
         if button ~= 'LeftButton' then return end
         f.pEnd = f.pEnd or {}
         f.pEnd.x, f.pEnd.y = select(4, f:GetPoint(1))
         f:StopMovingOrSizing()
         f:ClearAllPoints()
         local x = f.pOrigin.x + (f.pEnd.x - f.pStart.x)
         local y = f.pOrigin.y + (f.pEnd.y - f.pStart.y)
         f:SetPoint(f.pOrigin.point, f.pOrigin.anchor, f.pOrigin.relPoint, x, y)
        end)
    end


    local titleFrame =    _G[frameName .. "_TitleFrame"] or 
        CreateFrame("Frame", frameName .. "_TitleFrame", f)
    titleFrame:ClearAllPoints()
    titleFrame:SetPoint("TOPLEFT", 0, 0)
    titleFrame:SetHeight(bagDefaults.title_height)
    -- titleFrame:SetBackdrop(bagDefaults.backdrop_title)
    titleFrame:Show()

    local closebtn = _G[frameName .. "_CloseButton"] or 
        CreateFrame("Button", frameName .. "_CloseButton",
        titleFrame, "SackvilleBagCloseButton")
    closebtn:SetPoint("RIGHT", 0, 0)
    closebtn:Show()
    
    local minbtn = _G[frameName .. "_MinimizeButton"] or
        CreateFrame("Button", frameName .. "_MinimizeButton",
        titleFrame, "SackvilleBagMinimizeButton")
    minbtn:SetPoint("RIGHT", -20, 0)
    minbtn:Show()
    minbtn.bag = self
    
    local titleFontString = _G[frameName .. "_TitleText"] or 
        titleFrame:CreateFontString(frameName .. "_TitleText")
    titleFontString:SetFontObject("GameFontNormalLarge")
    titleFontString:SetText(bagName)
    titleFontString:ClearAllPoints()
    titleFontString:SetPoint("LEFT", 12, 0)
    titleFrame.text = titleFontString
    -- See http://www.wowwiki.com/API_FontInstance_SetFontObject

    titleFrame:SetWidth(titleFontString:GetStringWidth() + Sackville.Options.TITLE_EXTRA_WIDTH)
    f:SetWidth(titleFrame:GetWidth())

    f.titleFrame = titleFrame
    
    return f
end

function Sackville.Bag:SetTitle(text)
    self:GetFrame().titleFrame.text:SetText(text)
end

function Sackville.Bag:CreateQuickPanel()
    local qpanel = Sackville.Bag:CreateBagFrame(Sackville.version, PLAYER, true)
    moneyFontString = qpanel:CreateFontString("Sackville_money")
    moneyFontString:SetFontObject("GameFontWhite")
    moneyFontString:SetText(Sackville.GetMoneyString())
    moneyFontString:ClearAllPoints()
    -- moneyFontString:SetPoint("BOTTOMLEFT", 30, 10)
    moneyFontString:SetPoint("BOTTOMLEFT", 30, 5)
    
    local editSearch = _G["Sackville_qpanelSearch"] or
        CreateFrame("EditBox", "Sackville_qpanelSearch", qpanel, "SearchBoxTemplate")
    editSearch:SetPoint("TOPLEFT", 15, -30)
    editSearch:SetHeight(15)
    editSearch:SetWidth(qpanel:GetWidth() - 30)
    editSearch:SetScript("OnTextChanged", function (text)
        Sackville:SearchBoxUpdate(text) end)
    qpanel.searchBox = editSearch
    qpanel.money = moneyFontString
    qpanel:SetHeight(70)
    w = qpanel.money:GetStringWidth() + Sackville.Options.TITLE_EXTRA_WIDTH
    qpanel:SetWidth(w)
    qpanel.titleFrame:SetWidth(w)
    return qpanel
end

function Sackville.Bag:MinimizeClicked()  
    if self.name then
        self.minimized = not self.minimized
        Sackville.db.char.bagState[self.name].minimized = self.minimized
        self:LayoutItems()
        Sackville:LayoutBags(self)
    end
end

function Sackville.Bag:CreateBagControls()
    local controlFrame = AceGUI:Create("SimpleGroup")
    controlFrame:SetLayout("Flow")
    
    local btnMin = AceGUI:Create("SackvilleButton")
    btnMin:SetAutoWidth(true)
    btnMin:SetText("_")
    controlFrame:AddChild(btnMin)
    
    local btnClose = AceGUI:Create("SackvilleButton")
    btnClose:SetAutoWidth(true)
    btnClose:SetText("X")
    controlFrame:AddChild(btnClose)
    
    return controlFrame
end

function Sackville.Bag:LayoutItems()
    Sackville:Debug(1, "bag", "LayoutItems for bag: %s", self.name)
    local bagDefaults = Sackville.Options.bag
    local sectionDefaults = Sackville.Options.section
    local BUTTON_SIZE = Sackville.Options.BUTTON_SIZE
    local margin_top = Sackville.Options.section.margin_top
    
    local starting_height = self:GetHeight()
    local starting_width = self:GetWidth()
    
    self.state = {
        x_pos = bagDefaults.margin_left,
        y_pos = -bagDefaults.margin_top - Sackville.Options.section.margin_top,
        frame_x = bagDefaults.margin_left,
        frame_y = -bagDefaults.margin_top - Sackville.Options.section.margin_top,
        row = 0, col = 0,
        widest = 0
    }
    
    local minSections = {}

    for _, section in self:SectionIterator() do
        if self.minimized then
            section:HideAll()
        elseif section:IsShown() then
            local isMinimized = section:LayoutContents(self)
            if isMinimized then 
                table.insert(minSections, section) 
            elseif self.state.col ~= 0 then 
                self.state.x_pos = self.state.x_pos + sectionDefaults.padding_right
            end
        else
            section:HideAll(self.bagType)
        end
    end
    
    if self.state.col ~= 0 then
        self.state.x_pos = bagDefaults.margin_left
        self.state.y_pos = self.state.y_pos - BUTTON_SIZE
        self.state.y_pos = self.state.y_pos - margin_top
    end
    
    for _, section in ipairs(minSections) do
        local x, y = self:GetNextTitlePosition(section:GetTitleStringWidth())
        section:SetTitlePoint("TOPLEFT", x, y)
        section:ShowTitle()
    end
    
    self:SetWidth(math.max(self.min_width, self.state.widest))
    
    if self.minimized then
        self:SetMinHeight()
    else
        self:SetHeight(math.abs(self.state.frame_y) + bagDefaults.margin_bottom ) -- FIXME blows up with nil #1 arg
    end

    return true
end

function Sackville.Bag:GetNextItemPosition()
    local state = self.state
    -- Save the current position to be returned
    local x = state.x_pos
    local y = state.y_pos
    
    -- Change state the next position
    local BUTTON_SIZE = Sackville.Options.BUTTON_SIZE
    local bagDefaults = Sackville.Options.bag
    local margin_top = Sackville.Options.section.margin_top

    state.x_pos = state.x_pos + BUTTON_SIZE
    if state.x_pos > state.widest then 
        state.widest = state.x_pos + bagDefaults.margin_left 
    end
    
    if (state.x_pos + BUTTON_SIZE + bagDefaults.margin_right > self.max_width) then
        widest = self.max_width
        state.row = state.row + 1
        state.col = 0
        state.x_pos = bagDefaults.margin_left
        state.y_pos = state.y_pos - margin_top - BUTTON_SIZE
        state.frame_y = state.y_pos + margin_top
    else
        state.col = state.col + 1
        state.frame_y = state.y_pos - BUTTON_SIZE
    end
    
    state.frame_x = x + BUTTON_SIZE
    
    self.state = state
    return x, y
end

function Sackville.Bag:GetNextTitlePosition(widthNeeded)
    local BUTTON_SIZE = Sackville.Options.BUTTON_SIZE
    local bagDefaults = Sackville.Options.bag
    local margin_top = Sackville.Options.section.margin_top

    local x = self.state.x_pos
    local y = self.state.y_pos
    self.state.frame_y = y - bagDefaults.margin_bottom
    if (x + widthNeeded + bagDefaults.margin_right > self.max_width) then
        x = bagDefaults.margin_left
        y = y - margin_top
        self.state.frame_y = y - bagDefaults.margin_bottom
        self.state.y_pos = y
    end
    
    self.state.x_pos = x + widthNeeded + bagDefaults.margin_right
    if self.state.x_pos > self.state.widest then 
        self.state.widest = self.state.x_pos + bagDefaults.margin_left 
    end
    
    Sackville:Debug(1, "bag", "Got title position:(%d,%d), needed %d", x, y, widthNeeded)
    return x, y
end

function Sackville.Bag:SectionIterator()
    sectionSortCritera = { { key = "displayOrder", comparison = "<" } }
    return Sackville.Util.kpairs(self.sections, sectionSortCritera)
end

function Sackville.Bag:ReloadQuestItems()
    for _, section in self:SectionIterator() do
        section:ReloadQuestItems()
    end
end

function Sackville.Bag:Toggle()
    if (self:GetFrame():IsVisible()) then
        self:Hide()
    else
        self:Show()
    end
end

function Sackville.Bag:Show()
    self:LayoutItems()
    self:GetFrame():Show()
end

function Sackville.Bag:Hide()
    self:GetFrame():Hide()
end

function Sackville.Bag:HideSections()
    for _, section in self:SectionIterator() do
        section:Hide()
    end
end

function Sackville.Bag:ShowSections()
    for _, section in self:SectionIterator() do
        section:Show()
    end
end

function Sackville.Bag:HighlightSearchItems(text)
    for _, section in self:SectionIterator() do
        section:HighlightSearchItems(text)
    end
end

function Sackville.Bag:GetFrame()
    return self.frame
end

function Sackville.Bag:GetWidth()
    return self.frame:GetWidth()
end

function Sackville.Bag:GetHeight()
    return self.frame:GetHeight()
end

function Sackville.Bag:SetWidth(w)
    self.frame:SetWidth(w)
    self.frame.titleFrame:SetWidth(w)
end

function Sackville.Bag:SetHeight(h)
    self.frame:SetHeight(h)
end

function Sackville.Bag:SetMinHeight()
    self:SetHeight(self.frame.titleFrame:GetHeight())
end