local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")

Sackville.Config = {}
Sackville.Config.__index = Sackville.Config

local sprintf = string.format
local AceGUI = LibStub("AceGUI-3.0")
local _

local profile
local tabGroup
local txtBagSections = L["Bags"] .. " / " .. L["Sections"]

function Sackville.Config:CreateGUI()   
    profile = Sackville.Util.DeepCopy(Sackville:GetDBProfile())
    -- configFrame = AceGUI:Create("Frame")
    -- configFrame:SetCallback("OnClose", function(widget) 
        -- AceGUI:Release(widget) 
        -- Sackville.isConfigOpen = false
        -- end)
    -- configFrame:SetTitle("Sackville Configuration")
    -- configFrame:SetLayout("Fill")
    -- configFrame:SetPoint("TOPLEFT", 20, -20)
    
    tabGroup = AceGUI:Create("TabGroup")
    configFrame:AddChild(tabGroup)
    tabGroup:SetCallback("OnGroupSelected", TabGroup_OnGroupSelected)
    tabGroup:SetLayout("Fill")
    tabGroup:SetTabs(
    {
        -- {
            -- text = GENERAL,
            -- value = GENERAL
        -- },
        {
            text = txtBagSections,
            value = L["Bags"]
        },
        {   
            text = CATEGORIES,
            value = CATEGORIES
        }
    })
    tabGroup:SelectTab(L["Bags"])
    -- tabGroup:SelectTab(CATEGORIES)
    
    Sackville.configFrame = configFrame
    Sackville.configFrame.tabGroup = tabGroup
    -- Sackville.configTabs = tabGroup
end

function ShowBagConfig(bagConfig)
    Sackville:Debug(1, "config", "ShowBagConfig(%s)", bagConfig.name)
    local sfContents = AceGUI:Create("ScrollFrame")
    tabGroup:AddChild(sfContents)
    sfContents:SetLayout("List")

    local btnBack = AceGUI:Create("SackvilleButton")
    sfContents:AddChild(btnBack)
    btnBack:SetCallback("OnClick", BagBackButton_OnClick)
    btnBack:SetAutoWidth(true)
    btnBack:SetText(BACK)
    btnBack:SetPoint("RIGHT")
    
    local labHeader = AceGUI:Create("Label")
    sfContents:AddChild(labHeader)
    labHeader:SetFontObject(GameFontHighlightHuge)
    labHeader:SetText(bagConfig.name)
    
    local sgTopGroup = AceGUI:Create("SimpleGroup")
    sfContents:AddChild(sgTopGroup)
    sgTopGroup:SetLayout("Flow")
    
    local ddType = AceGUI:Create("Dropdown")
    sgTopGroup:AddChild(ddType)
    ddType:SetCallback("OnValueChanged", BagType_OnChange)
    ddType:SetUserData("bagConfig", bagConfig)
    ddType:SetLabel(L["Type"])
    ddType:SetList(Sackville.Options.bagTypeList ) 
    ddType:SetValue(bagConfig.bagType)
    ddType:SetWidth(90)

    local slideDefaultPriority = AceGUI:Create("Slider")
    sgTopGroup:AddChild(slideDefaultPriority)
    slideDefaultPriority:SetCallback("OnValueChanged", BagDefaultPrio_OnChange)
    slideDefaultPriority:SetUserData("bagConfig", bagConfig)
    slideDefaultPriority:SetSliderValues(0, 10, 1)
    slideDefaultPriority:SetValue(bagConfig.defaultPriority)
    slideDefaultPriority:SetLabel(sprintf("%s (%s)", L["Section priority"], DEFAULT))
    
    local igSectionList = AceGUI:Create("InlineGroup")
    sfContents:AddChild(igSectionList)
    igSectionList:SetTitle(L["Sections"])
    igSectionList:SetRelativeWidth(1)
    PopulateSectionList(igSectionList, bagConfig)
    
    local igSortCriteria = AceGUI:Create("InlineGroup")
    sfContents:AddChild(igSortCriteria)
    igSortCriteria:SetTitle(sprintf("%s (%s)", L["Sort criteria"], DEFAULT))
    igSortCriteria:SetRelativeWidth(1)
    PopulateSortCriteria(igSortCriteria, bagConfig.defaultSortCriteria)
    
    sfContents:FixScroll()
end

function PopulateSectionList(igSectionList, bagConfig)
    Sackville:Debug(1, "config", "PopulateSectionList(%s)", bagConfig.name)
    
    local allSectionOpts = {}
    for _, sectionOpts in SectionOptIterator(bagConfig) do
        table.insert(allSectionOpts, sectionOpts)
    end
    
    for index, sectionOpts in ipairs(allSectionOpts) do
        local sectionGroup = AceGUI:Create("SimpleGroup")
        igSectionList:AddChild(sectionGroup)
        sectionGroup:SetLayout("Flow")
        sectionGroup:SetRelativeWidth(1)
        
        local editSection = AceGUI:Create("EditBox")
        sectionGroup:AddChild(editSection)
        editSection:SetCallback("OnEnterPressed", SectionName_OnEnterPressed)
        editSection:SetUserData("sectionName", sectionOpts.name)
        editSection:SetUserData("bagConfig", bagConfig)
        editSection:SetText(sectionOpts.name)
        
        local iconUp = AceGUI:Create("SackvilleIcon")
        sectionGroup:AddChild(iconUp)
        iconUp:SetCallback("OnClick", BagSectionUpArrow_OnClick)
        iconUp:SetUserData("allSectionOpts", allSectionOpts)
        iconUp:SetUserData("bagConfig", bagConfig)
        iconUp:SetUserData("index", index)
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        sectionGroup:AddChild(iconDown)    
        iconDown:SetCallback("OnClick", BagSectionDownArrow_OnClick)
        iconDown:SetUserData("allSectionOpts", allSectionOpts)
        iconDown:SetUserData("bagConfig", bagConfig)
        iconDown:SetUserData("index", index)
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)

        local btnEdit = AceGUI:Create("SackvilleButton")
        sectionGroup:AddChild(btnEdit)
        btnEdit:SetCallback("OnClick", SectionEdit_OnClick)
        btnEdit:SetUserData("bagConfig", bagConfig)
        btnEdit:SetUserData("sectionName", sectionOpts.name)
        btnEdit:SetAutoWidth(true)
        btnEdit:SetText(L["Edit"])
        
        local btnRemove = AceGUI:Create("SackvilleButton")
        sectionGroup:AddChild(btnRemove)   
        btnRemove:SetCallback("OnClick", SectionRemove_OnClick)
        btnRemove:SetUserData("sectionName", sectionOpts.name)
        btnRemove:SetUserData("bagConfig", bagConfig)
        btnRemove:SetUserData("igSectionList", igSectionList)
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
    end

    local sectionGroup = AceGUI:Create("SimpleGroup")
    igSectionList:AddChild(sectionGroup)
    sectionGroup:SetLayout("Flow")
    sectionGroup:SetRelativeWidth(1)
    
    local ebNewSection = AceGUI:Create("EditBox")
    sectionGroup:AddChild(ebNewSection)
    
    local btnCreateSection = AceGUI:Create("SackvilleButton")
    sectionGroup:AddChild(btnCreateSection)        
    btnCreateSection:SetCallback("OnClick", NewSectionButton_OnClick)
    btnCreateSection:SetUserData("bagConfig", bagConfig)
    btnCreateSection:SetUserData("igSectionList", igSectionList)
    btnCreateSection:SetUserData("ebNewSection", ebNewSection)
    btnCreateSection:SetText(CALENDAR_CREATE)
    btnCreateSection:SetAutoWidth(true)
    -- Add empty line
    
    return igSectionList
end

function ShowSectionConfig(bagConfig, sectionName)
    local sectionOpts = bagConfig.sections[sectionName]
    Sackville:Debug(1, "config", "ShowSectionConfig(%s, %s)", 
        sectionOpts.name, bagConfig.name)
    local sfContents = AceGUI:Create("ScrollFrame")
    tabGroup:AddChild(sfContents)
    sfContents:SetLayout("List")
    
    local btnBack = AceGUI:Create("SackvilleButton")
    sfContents:AddChild(btnBack)
    btnBack:SetCallback("OnClick", SectionBackButton_OnClick)
    btnBack:SetUserData("bagConfig", bagConfig)
    btnBack:SetAutoWidth(true)
    btnBack:SetText(BACK)
    btnBack:SetPoint("RIGHT")
    
    local labHeader = AceGUI:Create("Label")
    sfContents:AddChild(labHeader)
    labHeader:SetFontObject(GameFontHighlightHuge)
    labHeader:SetText(sprintf("%s > %s", bagConfig.name, sectionOpts.name))
    
    local cbIsDefault = AceGUI:Create("CheckBox")
    sfContents:AddChild(cbIsDefault)
    cbIsDefault:SetCallback("OnValueChanged", SectionCbIsDefault_OnChange)
    cbIsDefault:SetUserData("sectionOpts", sectionOpts)
    cbIsDefault:SetLabel(L["Match all items"])
    cbIsDefault:SetValue(sectionOpts.alwaysMatch)
    
    local sgPriority = AceGUI:Create("SimpleGroup")
    sfContents:AddChild(sgPriority)
    sgPriority:SetLayout("Flow")
    sgPriority:SetFullWidth(true)
    
    local cbUseDefaultPrio = AceGUI:Create("CheckBox")
    sgPriority:AddChild(cbUseDefaultPrio)
    cbUseDefaultPrio:SetCallback("OnValueChanged", SectionCbUseDefaultPrio_OnChange)
    cbUseDefaultPrio:SetUserData("sectionOpts", sectionOpts)
    cbUseDefaultPrio:SetLabel(L["Use default priority"])
    cbUseDefaultPrio:SetValue(sectionOpts.useDefaultPriority)
    
    local slidePriority = AceGUI:Create("Slider")
    sgPriority:AddChild(slidePriority)
    cbUseDefaultPrio:SetUserData("slidePriority", slidePriority)
    slidePriority:SetCallback("OnValueChanged", SectionPrio_OnChange)
    slidePriority:SetUserData("bagConfig", bagConfig)
    slidePriority:SetUserData("sectionName", sectionOpts.name)
    slidePriority:SetSliderValues(0, 10, 1)
    -- local prio = sectionOpts.useDefaultPriority and bagConfig.defaultPriority
        -- or sectionOpts.priority
    -- Sackville:Debug(2, "config", "  section: %s", sectionOpts.name)
    -- Sackville:Debug(2, "config", "  useDefaultPriority: %s", tostring(sectionOpts.useDefaultPriority))
    -- Sackville:Debug(2, "config", "  bagConfig.defaultPriority: %s", tostring(bagConfig.defaultPriority))
    -- Sackville:Debug(2, "config", "  sectionOpts.priority: %s", tostring(sectionOpts.priority))
    -- Sackville:Debug(2, "config", "  Trying to set section prio %s", prio)
    slidePriority:SetValue(sectionOpts.priority)
    slidePriority:SetLabel(L["Section priority"])
    slidePriority:SetDisabled(sectionOpts.useDefaultPriority)
    
    local cbUseDefaultSorting = AceGUI:Create("CheckBox")
    sfContents:AddChild(cbUseDefaultSorting)
    cbUseDefaultSorting:SetCallback("OnValueChanged", 
        SectionCbUseDefaultSorting_OnChange)
    cbUseDefaultSorting:SetUserData("sectionOpts", sectionOpts)
    cbUseDefaultSorting:SetLabel(L["Use default sorting"])
    cbUseDefaultSorting:SetValue(sectionOpts.useDefaultSorting)
    
    local igSortCriteria = AceGUI:Create("InlineGroup")
    sfContents:AddChild(igSortCriteria)
    cbUseDefaultSorting:SetUserData("igSortCriteria", igSortCriteria)
    igSortCriteria:SetRelativeWidth(1)
    igSortCriteria:SetLayout("List")
    igSortCriteria:SetTitle(L["Sort criteria"])
    
    PopulateSortCriteria(igSortCriteria, sectionOpts.sortCriteria)

    -- Show categories

    local catListStatus = {}
    for _, key in ipairs(sectionOpts.categories) do
        catListStatus[key] = true
    end
    
    local catList = {}
    
    for catName,_ in pairs(profile.categories) do
        catList[catName] = catName
    end    
    
    local ddCategories = AceGUI:Create("SackvilleDropdown")
    sfContents:AddChild(ddCategories)
    ddCategories:SetCallback("OnValueChanged", SectionCatList_OnChange)
    ddCategories:SetUserData("categories", sectionOpts.categories )
    ddCategories:SetMultiselect(true)
    ddCategories:SetLabel(CATEGORIES)
    ddCategories:SetList(catList, nil, nil, catListStatus)
    
    sfContents:FixScroll()
end

function PopulateSortCriteria(container, sortCriteria)
   for index, criterion in ipairs(sortCriteria) do
        local sgCriterion = AceGUI:Create("SimpleGroup")
        container:AddChild(sgCriterion)
        sgCriterion:SetLayout("Flow")
        sgCriterion:SetRelativeWidth(1)
        
        local ddKey = AceGUI:Create("Dropdown")
        sgCriterion:AddChild(ddKey)
        ddKey:SetCallback("OnValueChanged", SortKey_OnChange)
        ddKey:SetUserData("index", index)
        ddKey:SetUserData("sortCriteria", sortCriteria)
        ddKey:SetList(Sackville.Options.itemSortKeyList)
        ddKey:SetValue(criterion.key)
        ddKey:SetLabel(L["Key"])
        ddKey:SetWidth(140)      

        local ddOper = AceGUI:Create("Dropdown")
        sgCriterion:AddChild(ddOper)
        ddOper:SetCallback("OnValueChanged", SortOper_OnChange)
        ddOper:SetUserData("index", index)
        ddOper:SetUserData("sortCriteria", sortCriteria)
        ddOper:SetList(Sackville.Options.itemSortOperatorList)
        ddOper:SetValue(criterion.comparison)
        ddOper:SetLabel(L["Order"])
        ddOper:SetWidth(140)  
        
        local iconUp = AceGUI:Create("SackvilleIcon")
        sgCriterion:AddChild(iconUp)
        iconUp:SetCallback("OnClick", SortRuleUpArrow_OnClick)
        iconUp:SetUserData("index", index)
        iconUp:SetUserData("container", container)
        iconUp:SetUserData("sortCriteria", sortCriteria)
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        sgCriterion:AddChild(iconDown)
        iconDown:SetCallback("OnClick", SortRuleDownArrow_OnClick)
        iconDown:SetUserData("index", index)
        iconDown:SetUserData("container", container)
        iconDown:SetUserData("sortCriteria", sortCriteria)
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)        

        local btnRemove = AceGUI:Create("SackvilleButton")
        sgCriterion:AddChild(btnRemove)
        btnRemove:SetCallback("OnClick", SortCriterionRemove_OnClick)
        btnRemove:SetUserData("index", index)
        btnRemove:SetUserData("sortCriteria", sortCriteria)
        btnRemove:SetUserData("container", container)
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
        
    end
    
    -- Empty value with create button
    local sgCriterion = AceGUI:Create("SimpleGroup")
    container:AddChild(sgCriterion)
    sgCriterion:SetLayout("Flow")
    sgCriterion:SetRelativeWidth(1)
    
    local ddKey = AceGUI:Create("Dropdown")
    sgCriterion:AddChild(ddKey)
    ddKey:SetList(Sackville.Options.itemSortKeyList)
    ddKey:SetLabel(L["Key"])
    ddKey:SetWidth(140)      

    local ddOper = AceGUI:Create("Dropdown")
    sgCriterion:AddChild(ddOper)
    ddOper:SetList(Sackville.Options.itemSortOperatorList)
    ddOper:SetLabel(L["Order"])
    ddOper:SetWidth(140)
    
    local btnSave = AceGUI:Create("SackvilleButton")
    sgCriterion:AddChild(btnSave)
    btnSave:SetCallback("OnClick", SaveSortCriterion_OnClick)
    btnSave:SetUserData("key", ddKey)
    btnSave:SetUserData("oper", ddOper)
    btnSave:SetUserData("container", container)
    btnSave:SetUserData("sortCriteria", sortCriteria)
    btnSave:SetAutoWidth(true)
    btnSave:SetText(SAVE)
end

function ShowBagList()
    Sackville:Debug(1, "config", "ShowBagList()")
    local sfContents = AceGUI:Create("ScrollFrame")
    sfContents:SetLayout("List")
    tabGroup:AddChild(sfContents)
    
   
    local labHeader = AceGUI:Create("Label")
    sfContents:AddChild(labHeader)
    labHeader:SetFontObject(GameFontHighlightHuge)
    labHeader:SetText(L["Bags"])
    -- Add a list of all current bags with buttons to reorder/delete
    -- local igBagList = AceGUI:Create("InlineGroup")
    -- igBagList:SetTitle(L["Bags"])
    -- igBagList:SetRelativeWidth(1)
    -- sfContents:AddChild(igBagList)
    
    -- Add controls to create a new bag
    local sgNewBagControls = AceGUI:Create("SimpleGroup")
    sgNewBagControls:SetLayout("Flow")        
    sfContents:AddChild(sgNewBagControls)
    
    local ebNewBagName = AceGUI:Create("EditBox")
    sgNewBagControls:AddChild(ebNewBagName)
    
    local btnCreateBag = AceGUI:Create("SackvilleButton")
    sgNewBagControls:AddChild(btnCreateBag)        
    btnCreateBag:SetCallback("OnClick", NewBag_OnClick)
    btnCreateBag:SetUserData("ebNewBagName", ebNewBagName)
    btnCreateBag:SetText(CALENDAR_CREATE)
    btnCreateBag:SetAutoWidth(true)
    
    local allBagOpts = {}
    for _, bagConfig in BagOptIterator() do
        table.insert(allBagOpts, bagConfig)
    end
    
    -- for index, bagConfig in BagOptIterator() do
    for index, bagConfig in ipairs(allBagOpts) do
        local bagGroup = AceGUI:Create("SimpleGroup")
        -- igBagList:AddChild(bagGroup)
        sfContents:AddChild(bagGroup)
        bagGroup:SetLayout("Flow")
        bagGroup:SetRelativeWidth(1)
        
        local editBagName = AceGUI:Create("EditBox")
        bagGroup:AddChild(editBagName)
        editBagName:SetCallback("OnEnterPressed", BagName_OnEnterPressed)
        editBagName:SetUserData("bagName", bagConfig.name)
        editBagName:SetText(bagConfig.name)
        -- editBagName:SetWidth(120)
               
        local iconUp = AceGUI:Create("SackvilleIcon")
        bagGroup:AddChild(iconUp)
        iconUp:SetCallback("OnClick", BagUpArrow_OnClick)
        iconUp:SetUserData("index", index)
        iconUp:SetUserData("allBagOpts", allBagOpts)
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        bagGroup:AddChild(iconDown)
        iconDown:SetUserData("index", index)
        iconDown:SetUserData("allBagOpts", allBagOpts)        
        iconDown:SetCallback("OnClick", BagDownArrow_OnClick)
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)
        
        local btnEditBag = AceGUI:Create("SackvilleButton")
        bagGroup:AddChild(btnEditBag)
        btnEditBag:SetCallback("OnClick", BagEditButton_OnClick)
        btnEditBag:SetUserData("bagConfig", bagConfig)
        btnEditBag:SetAutoWidth(true)
        btnEditBag:SetText(L["Edit"])
            
        local btnRemove = AceGUI:Create("SackvilleButton")
        bagGroup:AddChild(btnRemove)
        btnRemove:SetCallback("OnClick", BagRemoveButton_OnClick)
        btnRemove:SetUserData("bagName", bagConfig.name)
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
    end
    
    sfContents:FixScroll()
end

function ShowCategoryGroup(catName)
    Sackville:Debug(1, "config", "ShowCategoryGroup(%s)", catName)
    if catName then
        ShowCategory(catName)
    else
        ShowCategoryList()
    end
end

function ShowCategoryList()
    Sackville:Debug(1, "config", "ShowCategoryList()")
    local sfContents = AceGUI:Create("ScrollFrame")
    tabGroup:AddChild(sfContents)
    sfContents:SetLayout("List")

    local labHeader = AceGUI:Create("Label")
    sfContents:AddChild(labHeader)
    labHeader:SetFontObject(GameFontHighlightHuge)
    labHeader:SetText(CATEGORIES)
    
    -- Add controls to create a new category
    local sgNewCatControls = AceGUI:Create("SimpleGroup")
    sgNewCatControls:SetLayout("Flow")        

    local editNewCat = AceGUI:Create("EditBox")
    sgNewCatControls:AddChild(editNewCat)        
    
    local btnCreateCat = AceGUI:Create("SackvilleButton")
    sgNewCatControls:AddChild(btnCreateCat)
    btnCreateCat:SetCallback("OnClick", CatCreateButton_OnClick)
    btnCreateCat:SetUserData("editBox", editNewCat)
    btnCreateCat:SetText(CALENDAR_CREATE)
    btnCreateCat:SetAutoWidth(true)
    
    sfContents:AddChild(sgNewCatControls)
    
    -- Add a list of all categories with buttons to delete
    -- local igCatList = AceGUI:Create("InlineGroup")
    -- sfContents:AddChild(igCatList)
    -- igCatList:SetLayout("Flow")
    -- igCatList:SetRelativeWidth(1)
    
    local sgCatList = AceGUI:Create("SimpleGroup")
    sfContents:AddChild(sgCatList)
    sgCatList:SetLayout("Flow")
    sgCatList:SetRelativeWidth(1)

    local catList = {}
    for catName, _ in pairs(profile.categories) do
        table.insert(catList, catName)
    end
    table.sort(catList)
    
    for _, catName in ipairs(catList) do    
        local catGroup = AceGUI:Create("SimpleGroup")
        -- igCatList:AddChild(catGroup)
        sgCatList:AddChild(catGroup)
        catGroup:SetLayout("Flow")
        catGroup:SetRelativeWidth(.5)
        
        local editCatName = AceGUI:Create("EditBox")
        catGroup:AddChild(editCatName)
        editCatName:SetText(catName)
        editCatName:SetWidth(125)

        local btnEditCat = AceGUI:Create("SackvilleButton")
        catGroup:AddChild(btnEditCat)
        btnEditCat:SetCallback("OnClick", CatEditButton_OnClick)
        btnEditCat:SetUserData("catName", catName)
        btnEditCat:SetAutoWidth(true)
        btnEditCat:SetText(L["Edit"])
        
        local btnRemove = AceGUI:Create("SackvilleButton")
        catGroup:AddChild(btnRemove)
        btnRemove:SetCallback("OnClick", CatRemoveButton_OnClick)
        btnRemove:SetUserData("tabGroup", tabGroup)
        btnRemove:SetUserData("catName", catName)
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
    end   
    sfContents:FixScroll()
end

function ShowCategory(catName)
    Sackville:Debug(1, "config", "ShowCategory(%s)", catName)
    local sfContents = AceGUI:Create("ScrollFrame")
    tabGroup:AddChild(sfContents)
    sfContents:SetLayout("List")
    
    local btnClose = AceGUI:Create("SackvilleButton")
    sfContents:AddChild(btnClose)
    btnClose:SetCallback("OnClick", CatCloseButton_OnClick)
    btnClose:SetAutoWidth(true)
    btnClose:SetText(BACK)

    local labHeader = AceGUI:Create("Label")
    sfContents:AddChild(labHeader)
    labHeader:SetFontObject(GameFontHighlightHuge)
    labHeader:SetText(catName)
    
    -- local editName = AceGUI:Create("EditBox")
    -- sfContents:AddChild(editName)
    -- editName:SetLabel(CATEGORY)
    -- editName:SetText(catName)
    
    local igRules = AceGUI:Create("InlineGroup")
    sfContents:AddChild(igRules)
    igRules:SetLayout("List")
    igRules:SetTitle("Rules")
    igRules:SetRelativeWidth(1)
    
    -- List the rules
    Sackville:Debug(1, "config", "Listing rules for category: %s", catName)
    local catRules = profile.categories[catName]

    for index, rule in ipairs(catRules) do
        local sgRule = AceGUI:Create("SimpleGroup")    
        igRules:AddChild(sgRule)
        sgRule:SetRelativeWidth(1)
        sgRule:SetLayout("Flow")
        local negate = rule.negate or false
        local operation = rule.operation or L["AND"]
        local params = rule.params or {}
        local filterPath = string.gsub(rule.filterPath, "\001", ".")
        
        local sgRuleLabels = AceGUI:Create("SimpleGroup")
        sgRule:AddChild(sgRuleLabels)
        sgRuleLabels:SetLayout("List")
        
        local labRule = AceGUI:Create("Label")
        sgRuleLabels:AddChild(labRule)
        labRule:SetWidth(250)
        local txtOp
        if index == 1 then 
            txtOp = "" 
        else 
            txtOp = operation .. " "
        end
        local txtNeg = negate and (L["NOT"] .. " ") or ""
        local labelText = sprintf("%s%s%s",
            txtOp, txtNeg, filterPath)
        labRule:SetText(labelText)
        
        if params then
            for key, value in pairs(params) do
                labParam = AceGUI:Create("Label")
                sgRuleLabels:AddChild(labParam)
                labParam:SetText(sprintf("[%s] = %s", key, value))
            end
        end
        
        local iconUp = AceGUI:Create("SackvilleIcon")
        sgRule:AddChild(iconUp)
        iconUp:SetCallback("OnClick", RuleUpArrow_OnClick)
        iconUp:SetUserData("catRules", catRules)
        iconUp:SetUserData("index", index)
        -- iconUp:SetUserData("tabGroup", tabGroup)
        iconUp:SetUserData("catName", catName)
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        sgRule:AddChild(iconDown)
        iconDown:SetCallback("OnClick", RuleDownArrow_OnClick)
        iconDown:SetUserData("catRules", catRules)
        iconDown:SetUserData("index", index)    
        -- iconDown:SetUserData("tabGroup", tabGroup)
        iconDown:SetUserData("catName", catName)        
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)
        
        local btnEdit = AceGUI:Create("SackvilleButton")
        sgRule:AddChild(btnEdit)        
        btnEdit:SetCallback("OnClick", RuleEditButton_OnClick)
        btnEdit:SetAutoWidth(true)
        btnEdit:SetText(L["Edit"])
        btnEdit:SetUserData("rule", rule)
        -- btnEdit:SetUserData("tabGroup", tabGroup)
        btnEdit:SetUserData("catName", catName)
        
        local btnRemove = AceGUI:Create("SackvilleButton")
        sgRule:AddChild(btnRemove)
        btnRemove:SetCallback("OnClick", RuleRemoveButton_OnClick)
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
        btnRemove:SetUserData("ruleIndex", index)
        btnRemove:SetUserData("catRules", catRules)
        btnRemove:SetUserData("rule", rule)
        -- btnRemove:SetUserData("tabGroup", tabGroup)
        btnRemove:SetUserData("catName", catName)
    end
    
    -- local sgNewRule = AceGUI:Create("SimpleGroup")
    -- igRules:AddChild(sgNewRule)
    -- sgNewRule:SetLayout("Flow")
       
    local btnCreateRule = AceGUI:Create("SackvilleButton")
    sfContents:AddChild(btnCreateRule)
    btnCreateRule:SetCallback("OnClick", RuleCreateButton_OnClick)
    btnCreateRule:SetUserData("tabGroup", tabGroup)
    btnCreateRule:SetUserData("catName", catName)
    btnCreateRule:SetUserData("catRules", catRules)
    btnCreateRule:SetAutoWidth(true)
    btnCreateRule:SetText(CALENDAR_CREATE)
end

function ShowRuleDetails(rule, catName) 
    Sackville:Debug(1, "config", "ShowRuleDetails(rule, %s)", catName)
    rule.negate = rule.negate or false
    rule.operation = rule.operation or L["AND"]
    rule.params = rule.params or {}
    
    local sfContents = AceGUI:Create("ScrollFrame")
    tabGroup:AddChild(sfContents)
    sfContents:SetUserData("tabGroup", tabGroup)
    sfContents:SetUserData("rule", rule)
    sfContents:SetUserData("catName", catName)
    sfContents:SetLayout("List")
    sfContents:SetFullHeight(true)
    
    local btnBack = AceGUI:Create("SackvilleButton")
    btnBack:SetAutoWidth(true)
    btnBack:SetText(BACK)
    btnBack:SetCallback("OnClick", RuleBackButton_OnClick)
    sfContents:AddChild(btnBack)
    
    -- local labTitle = AceGUI:Create("Label")
    -- sfContents:AddChild(labTitle)
    -- labTitle:SetText(L["Editing rule for category:"] .. " " .. catName)
    -- labTitle:SetFontObject(GameFontNormalLarge)
    -- labTitle:SetFullWidth(true)
   
    local ddOper = AceGUI:Create("Dropdown")
    sfContents:AddChild(ddOper)
    ddOper:SetCallback("OnValueChanged", RuleOperator_OnChange)
    ddOper:SetList({ [L["AND"]] = "AND", [L["OR"]] = "OR" } )
    ddOper:SetValue(rule.operation)
    ddOper:SetLabel(L["Operator"])
    ddOper:SetWidth(90)

    local cbNegate = AceGUI:Create("CheckBox")
    sfContents:AddChild(cbNegate)
    cbNegate:SetCallback("OnValueChanged", RuleNegation_OnChange)
    cbNegate:SetValue(rule.negate)
    cbNegate:SetLabel(L["Negate"])
    cbNegate:SetWidth(90)
    
    local sgRuleCategories = AceGUI:Create("SimpleGroup")
    sfContents:AddChild(sgRuleCategories)
    sgRuleCategories:SetUserData("rule", rule)
    addRuleCategories(sgRuleCategories, nil, rule.filterPath, 
        Sackville.allFilters, true)
      
end

-- Populate the GUI container with all of the rules in the category 
function addRuleCategories(sgRuleCategories, parentPath, remainingPath, filters, firstNode)
    group, elements = strsplit("\001", remainingPath, 2)
   
    local ddCat = AceGUI:Create("Dropdown")
    sgRuleCategories:AddChild(ddCat)
    local filterList = makeFilterDropdownList(filters)
    ddCat:SetCallback("OnValueChanged", RuleCategory_OnChange)
    ddCat:SetUserData("parentPath", parentPath)
    ddCat:SetList(filterList)
    ddCat:AddItem("", "")
    if group ~= "" then
        ddCat:SetValue(group)
    else
        ddCat:SetValue("")
    end
    -- ddCat:SetWidth(140)
    if firstNode then 
        ddCat:SetLabel(CATEGORY)
    else
        ddCat:SetLabel(SUBCATEGORY)
    end
    
    local myPath
    if parentPath then
        myPath = parentPath .. "\001" .. group
    else
        myPath = group
    end
    
    if elements then
        addRuleCategories(sgRuleCategories, myPath, elements, 
            filters[group].children, false)
    elseif group ~= "" then
        local filterList = makeFilterDropdownList(filters[group].children)
        Sackville:Debug(2, "config", 
            "No more filterPath elements. Last group: %s, filterList count: %d",
            group, #filterList)
        if Sackville.Util.TableHasValues(filterList) then
            local ddSubCat = AceGUI:Create("Dropdown")
            ddSubCat:SetCallback("OnValueChanged", RuleCategory_OnChange)
            ddSubCat:SetUserData("parentPath", myPath)
            ddSubCat:SetList(filterList)
            ddSubCat:AddItem("", "")
            -- ddSubCat:SetWidth(140)
            ddSubCat:SetLabel(SUBCATEGORY)
            sgRuleCategories:AddChild(ddSubCat)
        end
        
        if filters[group].options then
            local params = sgRuleCategories:GetUserData("rule").params
            local controls = filters[group].options.controls
            Sackville:Debug(2, "config", "filter group %s has options", group)
            for _, control in ipairs(controls) do
                if control.inputType == "Dropdown" then
                    local ddOpt = AceGUI:Create("Dropdown")
                    sgRuleCategories:AddChild(ddOpt)
                    ddOpt:SetCallback("OnValueChanged", RuleParamChanged)
                    ddOpt:SetUserData("rule", sgRuleCategories:GetUserData("rule"))
                    ddOpt:SetUserData("paramName", control.paramName)
                    ddOpt:SetLabel(control.label)
                    ddOpt:SetList({})
                    for _, value in ipairs(control.values) do
                        ddOpt:AddItem(value, value)
                    end
                    ddOpt:SetText(params[control.paramName])
                    
                elseif control.inputType == "EditBox" then
                    local ebOpt = AceGUI:Create("EditBox")
                    sgRuleCategories:AddChild(ebOpt)
                    ebOpt:SetCallback("OnEnterPressed", RuleParamChanged)
                    ebOpt:SetUserData("rule", sgRuleCategories:GetUserData("rule"))
                    ebOpt:SetUserData("paramName", control.paramName)
                    ebOpt:SetLabel(control.label)
                    ebOpt:SetText(params[control.paramName])
                    if control.maxLetters then
                        ebOpt:SetMaxLetters(tonumber(control.maxLetters))
                    end
                end
            end
        end
    end
end

function makeFilterDropdownList(filters)
    Sackville:Debug(1, "config", "makeFilterDropdownList(filters)")
    local filterList = {}
    for name, _ in pairs(filters) do
        filterList[name] = name
    end
    return filterList
end

function MakeTree()
    Sackville:Debug(1, "config", "MakeTree()")
    local tree = {}
    local bags = {
        text = L["Bags"] .. " / " .. L["Sections"],
        value = L["Bags"],
        children = { }
    }
       
    for _, bagConfig in BagOptIterator() do
        local bagNode = { text = bagConfig.name, value = bagConfig.name, children = {} }
        
        for _, sectionOpts in SectionOptIterator(bagConfig) do            
            local sectionNode = { text = sectionOpts.name, value = sectionOpts.name }
            table.insert(bagNode.children, sectionNode)
        end
        table.insert(bags.children, bagNode)
    end
    
    local categories = {
        text = CATEGORIES,
        value = CATEGORIES,
        -- children = { }
    }
    
    -- local catList = {}
    -- for catName, _ in pairs(profile.categories) do
        -- table.insert(catList, catName)
    -- end
    -- table.sort(catList)
    -- for _, catName in ipairs(catList) do
        -- table.insert(categories.children, { text = catName, value = catName })
    -- end
    
    table.insert(tree, bags)
    table.insert(tree, categories)
    return tree
end

function BagOptIterator()
    local bagSortOptions = { {key = "displayOrder", comparison = "<" } }    
    return Sackville.Util.kpairs(profile.bagConfig, bagSortOptions)
end

function SectionOptIterator(bagConfig)
    sectionSortCritera = { { key = "displayOrder", comparison = "<" } }
    return Sackville.Util.kpairs(bagConfig.sections, sectionSortCritera)
end

-- EVENTS

function TabGroup_OnGroupSelected(tabGroup, event, tabName)
    Sackville:Debug(1, "config", "EVENT: TabGroup_OnGroupSelected: %s", tabName)
    -- Sackville:Print("Selected tab: " .. tabName)
    tabGroup:ReleaseChildren()
    if tabName == GENERAL then
    
    elseif tabName == L["Bags"] then
        ShowBagList()
    elseif tabName == CATEGORIES then
        ShowCategoryList()
    end
end

-- Bag list events

function BagName_OnEnterPressed(editBox, event, newName)
    Sackville:Debug(1, "config", "EVENT: BagName_OnEnterPressed: %s", newName)
    local bagName = editBox:GetUserData("bagName")
    local bagConfig = profile.bagConfig
    if newName ~= bagName then
        if bagConfig[newName] then
            Sackville:Print(L["That name is already in use."])
            editBox:SetText(bagName)
        else
            bagConfig[bagName].name = newName
            bagConfig[newName] = bagConfig[bagName]
            bagConfig[bagName] = nil
            editBox:SetUserData("bagName", newName)
            
        end
    end
end

function NewBag_OnClick(button)
    local bagName = button:GetUserData("ebNewBagName"):GetText()
    
    if Sackville:GetBag(bagName) then
        Sackville:Print(L["That name is already in use."])
        button:GetUserData("ebNewBagName"):SetText(bagName)
    else
        local bagConfig = profile.bagConfig
        local bagCount = Sackville.Util.GetTableCount(bagConfig)
        bagConfig[bagName] = {
            name = bagName,
            displayOrder = bagCount + 1,
            bagType = PLAYER,
            defaultPriority = 1,
            defaultSortCriteria = {},
            sections = {}
        }
        
        tabGroup:ReleaseChildren()
        ShowBagList()
    end
    
end

function BagEditButton_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagEditButton_OnClick")
    local bagConfig = button:GetUserData("bagConfig") 
    tabGroup:ReleaseChildren()
    ShowBagConfig(bagConfig)
end

function BagRemoveButton_OnClick(button)
    local bagConfig = profile.bagConfig
    local bagName = button:GetUserData("bagName")
    bagConfig[bagName] = nil
    tabGroup:ReleaseChildren()
    ShowBagList()
    
end

function BagBackButton_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagBackButton_OnClick")
    tabGroup:ReleaseChildren()
    ShowBagList()
end

function BagUpArrow_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagUpArrow_OnClick")
    local index = button:GetUserData("index")
    local allBagOpts = button:GetUserData("allBagOpts")

    if allBagOpts[index-1] then
        Sackville:Debug(1, "config", "swapping bag order")
        allBagOpts[index].displayOrder, allBagOpts[index-1].displayOrder = 
            allBagOpts[index-1].displayOrder, allBagOpts[index].displayOrder
        tabGroup:ReleaseChildren()
        ShowBagList()
        PlaySound("UChatScrollButton")
    end
    
end

function BagDownArrow_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagDownArrow_OnClick")
    local index = button:GetUserData("index")
    local allBagOpts = button:GetUserData("allBagOpts")

    if allBagOpts[index+1] then
        Sackville:Debug(1, "config", "swapping bag order")
        allBagOpts[index].displayOrder, allBagOpts[index+1].displayOrder = 
            allBagOpts[index+1].displayOrder, allBagOpts[index].displayOrder
        tabGroup:ReleaseChildren()
        ShowBagList()
        PlaySound("UChatScrollButton")
    end
    
end

-- Bag config events

function BagType_OnChange(ddType, event, value)
    local bagConfig = ddType:GetUserData("bagConfig")
    bagConfig.bagType = value
end

function BagDefaultPrio_OnChange(slider, event, value)
    local bagConfig = slider:GetUserData("bagConfig")
    bagConfig.defaultPriority = value
end

function SectionName_OnEnterPressed(editBox)
    local bagConfig = editBox:GetUserData("bagConfig")
    local sectionName = editBox:GetUserData("sectionName")
    local newName = editBox:GetText()
    
    if newName ~= sectionName then
        if bagConfig.sections[newName] then
            Sackville:Print(L["That name is already in use."])
            editBox:SetText(sectionName)            
        else
            bagConfig.sections[newName] = bagConfig.sections[sectionName]
            bagConfig.sections[newName].name = newName
            bagConfig.sections[sectionName] = nil
            editBox:SetUserData("sectionName", newName)    
        end
    end
    
end

function BagSectionUpArrow_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagSectionUpArrow_OnClick")
    local index = button:GetUserData("index")
    local allSectionOpts = button:GetUserData("allSectionOpts")
    local bagConfig = button:GetUserData("bagConfig")
    
    if allSectionOpts[index-1] then
        Sackville:Debug(1, "config", "swapping section order")
        allSectionOpts[index].displayOrder, allSectionOpts[index-1].displayOrder =
            allSectionOpts[index-1].displayOrder, allSectionOpts[index].displayOrder
        tabGroup:ReleaseChildren()
        ShowBagConfig(bagConfig)
        PlaySound("UChatScrollButton")
    end
    
end

function BagSectionDownArrow_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: BagSectionDownArrow_OnClick")
    local index = button:GetUserData("index")
    local allSectionOpts = button:GetUserData("allSectionOpts")
    local bagConfig = button:GetUserData("bagConfig")
    
    if allSectionOpts[index+1] then
        Sackville:Debug(1, "config", "swapping section order")
        allSectionOpts[index].displayOrder, allSectionOpts[index+1].displayOrder =
            allSectionOpts[index+1].displayOrder, allSectionOpts[index].displayOrder
        tabGroup:ReleaseChildren()
        ShowBagConfig(bagConfig)
        PlaySound("UChatScrollButton")
    end
    
end

function SectionEdit_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: SectionEdit_OnClick")
    local bagConfig = button:GetUserData("bagConfig")
    local sectionName = button:GetUserData("sectionName")
    tabGroup:ReleaseChildren()
    ShowSectionConfig(bagConfig, sectionName)
end

function SectionRemove_OnClick(button)
    local bagConfig = button:GetUserData("bagConfig")
    local sectionName = button:GetUserData("sectionName")
    local igSectionList = button:GetUserData("igSectionList")

    
    bagConfig.sections[sectionName] = nil
    igSectionList:ReleaseChildren()
    PopulateSectionList(igSectionList, bagConfig)
    
end

function SectionBackButton_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: SectionBack_OnClick")
    local bagConfig = button:GetUserData("bagConfig")
    tabGroup:ReleaseChildren()
    ShowBagConfig(bagConfig)
end

function NewSectionButton_OnClick(button)
    local bagConfig = button:GetUserData("bagConfig")
    local igSectionList = button:GetUserData("igSectionList")
    local sectionName = button:GetUserData("ebNewSection"):GetText()
    
    if bagConfig.sections[sectionName] then
        Sackville:Print(L["That name is already in use."])
    else
        bagConfig.sections[sectionName] = {
            name = sectionName,
            displayOrder = Sackville.Util.GetTableCount(bagConfig.sections) + 1,
            priority = 1,
            useDefaultPriority = true,
            useDefaultSorting = true,
            categories = {},
            sortCriteria = {},
            showEmptySlot = false,
            alwaysShow = false
        }
        
        igSectionList:ReleaseChildren()
        PopulateSectionList(igSectionList, bagConfig)
    end
end

function SaveSortCriterion_OnClick(button)
    local key = button:GetUserData("key").value
    local oper = button:GetUserData("oper").value
    local sortCriteria = button:GetUserData("sortCriteria")
    local container = button:GetUserData("container")
    Sackville:Debug(2, "config", "SaveSortCriterion_OnClick()")
    if  key == nil or key == "" or oper == nil or oper == "" then
        Sackville:Print(L["Please provide missing values."])
    else
        table.insert(sortCriteria, { ["key"] = key, ["comparison"] = oper } )
        container:ReleaseChildren()
        PopulateSortCriteria(container, sortCriteria)
    end
end

function SortCriterionRemove_OnClick(button)
    local index = button:GetUserData("index")
    local sortCriteria = button:GetUserData("sortCriteria")
    local container = button:GetUserData("container")
    
    table.remove(sortCriteria, index)
    container:ReleaseChildren()
    PopulateSortCriteria(container, sortCriteria)
end

-- Section config events

function SectionCbIsDefault_OnChange(widget, event, value)
    local sectionOpts = widget:GetUserData("sectionOpts")
    sectionOpts.alwaysMatch = value
    
end

function SectionCbUseDefaultPrio_OnChange(widget, event, value)
    Sackville:Debug(2, "config", "Use default priority?: " .. tostring(value))
    local sectionOpts = widget:GetUserData("sectionOpts")
    sectionOpts.useDefaultPriority = value
    
    local slidePriority = widget:GetUserData("slidePriority")
    slidePriority:SetDisabled(value)
    
end

function SectionPrio_OnChange(widget, event, value)
    local sectionName = widget:GetUserData("sectionName")
    local bagConfig = widget:GetUserData("bagConfig")
    bagConfig.sections[sectionName].priority = value
    
end

function SectionCbUseDefaultSorting_OnChange(widget, event, value)
    local sectionOpts = widget:GetUserData("sectionOpts")
    sectionOpts.useDefaultSorting = value
    
end

function SortRuleUpArrow_OnClick(button)
    local index = button:GetUserData("index")
    local container = button:GetUserData("container")
    local sortCriteria = button:GetUserData("sortCriteria")
    
    if sortCriteria[index-1] then
        sortCriteria[index], sortCriteria[index-1] =
            sortCriteria[index-1], sortCriteria[index]
        container:ReleaseChildren()
        PopulateSortCriteria(container, sortCriteria)
        
    end
end

function SortRuleDownArrow_OnClick(button)
    local index = button:GetUserData("index")
    local container = button:GetUserData("container")
    local sortCriteria = button:GetUserData("sortCriteria")
    
    if sortCriteria[index+1] then
        sortCriteria[index], sortCriteria[index+1] =
            sortCriteria[index+1], sortCriteria[index]
        container:ReleaseChildren()
        PopulateSortCriteria(container, sortCriteria)
        
    end
end

function SortKey_OnChange(ddKey, event, value)
    local index = ddKey:GetUserData("index")
    local sortCriteria = ddKey:GetUserData("sortCriteria")
    Sackville:Debug(2, "config", "Changed key to %s", value)
    -- Sackville:Print("Changing key to: " .. value)
    sortCriteria[index].key = value
    
end

function SortOper_OnChange(ddOper, event, value)
    local index = ddOper:GetUserData("index")
    local sortCriteria = ddOper:GetUserData("sortCriteria")
    Sackville:Debug(2, "config", "Changed operator to %s", value)
    -- Sackville:Print("Changing oper to: " .. value)
    sortCriteria[index].comparison = value
    
end

function SectionCatList_OnChange(widget, event, value, checked)
    Sackville:Debug(2, "config", "SectionCatList_OnChange(%s, %s, %s, %s)",
        tostring(ddCat), tostring(event), tostring(value), tostring(checked))
    
    local categories = widget:GetUserData("categories")
    
    if checked then
        table.insert(categories, value)
    else
        for index, catName in ipairs(categories) do
            if catName == value then
                table.remove(categories, index)
                break
            end 
        end
    end
    
    
end

-- Category list events

function RuleEditButton_OnClick(widget, event, button)
Sackville:Debug(1, "config", "EVENT: RuleEditButton_OnClick")
    local rule = widget:GetUserData("rule")
    local catName = widget:GetUserData("catName")
    tabGroup:ReleaseChildren()
    ShowRuleDetails(rule, catName)
end

function RuleBackButton_OnClick(widget, event, button)
    Sackville:Debug(1, "config", "EVENT: RuleBackButton_OnClick")
    local catName = widget.parent:GetUserData("catName")
    tabGroup:ReleaseChildren()
    ShowCategory(catName)
end

function RuleNegation_OnChange(widget, event, value)
    Sackville:Debug(1, "config", "EVENT: RuleNegation_OnChange")
    local rule = widget.parent:GetUserData("rule")
    rule.negate = value   
    widget.parent:SetUserData("rule", rule)
    
end

function RuleOperator_OnChange(widget, event, value)
    Sackville:Debug(1, "config", "EVENT: RuleOperator_OnChange")
    local rule = widget.parent:GetUserData("rule")
    rule.operation = value   
    widget.parent:SetUserData("rule", rule)
    
end

function RuleCategory_OnChange(widget, event, value)
    Sackville:Debug(1, "config", "EVENT: RuleCategory_OnChange")
    local sgRuleCategories = widget.parent.parent
    local rule = sgRuleCategories:GetUserData("rule")
    local catName = sgRuleCategories:GetUserData("catName")

    local parentPath = widget:GetUserData("parentPath")
    local filterPath
    if parentPath then
        if value ~= "" then
            filterPath = parentPath .. "\001" .. value
        else
            filterPath = parentPath
        end
    else
        filterPath = value
    end
    rule.filterPath = filterPath
    sgRuleCategories:SetUserData("rule", rule)
    tabGroup:ReleaseChildren()
    ShowRuleDetails(rule, catName)
    
end

function RuleRemoveButton_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: RuleRemoveButton_OnClick")
    local catName = widget:GetUserData("catName")
    local catRules = widget:GetUserData("catRules")
    local ruleIndex = widget:GetUserData("ruleIndex")
    table.remove(catRules, ruleIndex)

    tabGroup:ReleaseChildren()
    ShowCategory(catName)   
        
end

function RuleCreateButton_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: RuleCreateButton_OnClick")
    local catName = widget:GetUserData("catName")
    local catRules = widget:GetUserData("catRules")
    
    local rule = {
        negate = false,
        operation = "AND",
        filterPath = "",
        params = {}
    }
    
    table.insert(catRules, rule)
    tabGroup:ReleaseChildren()
    ShowRuleDetails(rule, catName)
end

function RuleUpArrow_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: RuleUpArrow_OnClick")
    local catName = widget:GetUserData("catName")
    local index = widget:GetUserData("index")
    local catRules = widget:GetUserData("catRules", catRules)

    if catRules[index-1] then
        catRules[index], catRules[index-1] = catRules[index-1], catRules[index]
        tabGroup:ReleaseChildren()
        ShowCategory(catName)
        PlaySound("UChatScrollButton")
    end
    
end

function RuleDownArrow_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: RuleDownArrow_OnClick")
    local catName = widget:GetUserData("catName")
    local index = widget:GetUserData("index")
    local catRules = widget:GetUserData("catRules", catRules)

    if catRules[index+1] then
        catRules[index], catRules[index+1] = catRules[index+1], catRules[index]        
        tabGroup:ReleaseChildren()
        ShowCategory(catName)    
        PlaySound("UChatScrollButton")
    end
    
end

function RuleParamChanged(widget, event, value)
    Sackville:Debug(1, "config", "RuleParamChanged(%s)", value)
    local params = widget:GetUserData("rule").params
    local paramName = widget:GetUserData("paramName")
    params[paramName] = value
    
end

function CatCreateButton_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: CatCreateButton_OnClick")
    local catName = widget:GetUserData("editBox"):GetText()
    
    if not profile.categories[catName] then
        profile.categories[catName] = {}
        tabGroup:ReleaseChildren()
        -- tabGroup:SetTree(MakeTree())
        -- tabGroup:SelectByPath(CATEGORIES .. "\001" .. catName)
        ShowCategory(catName)
    end
end

function CatEditButton_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: CatEditButton_OnClick")
    local catName = button:GetUserData("catName")
    tabGroup:ReleaseChildren()
    ShowCategory(catName)
end

function CatRemoveButton_OnClick(widget)
    Sackville:Debug(1, "config", "EVENT: CatRemoveButton_OnClick")
    local catName = widget:GetUserData("catName")
    profile.categories[catName] = nil
    tabGroup:ReleaseChildren()
    ShowCategoryList()
    
end

function CatCloseButton_OnClick(button)
    Sackville:Debug(1, "config", "EVENT: CatCloseButton_OnClick")
    tabGroup:ReleaseChildren()
    ShowCategoryList()
end

-- Other

function CommitChanges()
    Sackville:Debug(1, "config", "CommitChanges()")
    Sackville:SetDBProfile(profile)
    Sackville:OnBagConfigUpdate()
end

function RevertChanges()
    Sackville:Debug(1, "config", "RevertChanges()")
    profile = Sackville.Util.DeepCopy(Sackville:GetDBProfile())
    tabGroup:ReleaseChildren()
    tabGroup:SelectTab(txtBagSections)
    ShowBagList()        
end

function RestoreDefaultSettings()
    Sackville:Debug(1, "config", "RestoreDefaultSettings()") 
    Sackville:ResetProfile()
    Sackville:OnBagConfigUpdate()
end

function RefreshUI()
    Sackville:Debug(1, "config", "RefreshUI()")
    tabGroup:ReleaseChildren()
    tabGroup:SelectTab(txtBagSections)
    ShowBagList()    
end


configFrame = AceGUI:Create("SimpleGroup")
configFrame:SetLayout("Fill")

-- Sackville.panel = CreateFrame( "Frame", "SackvillePanel", UIParent );
Sackville.panel = configFrame.frame
Sackville.panel.name = "Sackville"
Sackville.panel.okay = CommitChanges
Sackville.panel.cancel = RevertChanges
Sackville.panel.default = RestoreDefaultSettings
Sackville.panel.refresh = RefreshUI
InterfaceOptions_AddCategory(Sackville.panel)

-- configFrame = AceGUI:Create("SimpleGroup")
-- configFrame:SetLayout("Fill")
-- configFrame.frame:SetParent(Sackville.panel)