local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
local B = LibStub("LibBabble-Inventory-3.0")
local BL = B:GetLookupTable()
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")

Sackville.Item = {}
Sackville.Item.__index = Sackville.Item
local sprintf = string.format

local equippedSlot = {
    INVTYPE_AMMO = 0,
    INVTYPE_HEAD = 1,
    INVTYPE_NECK = 2,
    INVTYPE_SHOULDER = 3,
    INVTYPE_BODY = 4,
    INVTYPE_CHEST = 5,
    INVTYPE_ROBE = 5,
    INVTYPE_WAIST = 6,
    INVTYPE_LEGS = 7,
    INVTYPE_FEET = 8,
    INVTYPE_WRIST = 9,
    INVTYPE_HAND = 10,
    INVTYPE_FINGER = 11,
    INVTYPE_TRINKET = 13,
    INVTYPE_CLOAK = 15,
    INVTYPE_WEAPON = 16,
    INVTYPE_SHIELD = 17,
    INVTYPE_2HWEAPON = 16,
    INVTYPE_WEAPONMAINHAND = 16,
    INVTYPE_WEAPONOFFHAND = 17,
    INVTYPE_HOLDABLE = 17,
    INVTYPE_RANGED = 18,
    INVTYPE_THROWN = 18,
    INVTYPE_RANGEDRIGHT = 18,
    INVTYPE_RELIC = 18,
    INVTYPE_TABARD = 19,
    INVTYPE_BAG = 20,
    INVTYPE_QUIVER = 21,
}

local function UpdateItemButtonCooldown(item)
    local cooldown = _G[item.button:GetName().."Cooldown"]
    local start, duration, enable = GetContainerItemCooldown(item.bagid, item.slot)
    CooldownFrame_SetTimer(cooldown, start, duration, enable);
    if ( duration > 0 and enable == 0 ) then
        SetItemButtonTextureVertexColor(item.button, 0.4, 0.4, 0.4);
    else
        SetItemButtonTextureVertexColor(item.button, 1, 1, 1);
    end
end

local function SetQualityOverlay(item)
-- This will overlay a frame with a colored border over the button.
    local basename = "SackvilleItemButton" .. item.bagslot .. "_quality"
    local button = item.button
    button.qualityFrame = _G[basename]
        or CreateFrame("Frame", basename, button)    
    button.qualityFrame.texture = _G[basename .. '_texture']
        or button.qualityFrame:CreateTexture(basename .. '_texture', 'OVERLAY')
    
    if (item:IsQuestItem()) then
        local questTexture
        if item.isActive then
            questTexture = TEXTURE_ITEM_QUEST_BORDER
        else
            questTexture = TEXTURE_ITEM_QUEST_BANG
        end
        
        button.qualityFrame.texture:SetTexture(questTexture)
        button.qualityFrame:Show()    
    else
        if item.quality then -- I shouldn't need to check, but it blew up on me one time
            button.qualityFrame.texture:SetTexture(sprintf(
            'Interface\\Addons\\Sackville\\Textures\\UI-Icon-Quality%sBorder.blp',
            item.quality))
            if (item.quality == nil) or (item.quality < 2) or (item.quality > 7) then 
                button.qualityFrame:Hide()
            else
                button.qualityFrame:Show()
            end
        else
            button.qualityFrame:Hide()
        end
    end
    
    button.qualityFrame.texture:SetBlendMode('BLEND')
    button.qualityFrame.texture:SetAllPoints()
    button.qualityFrame:SetAllPoints()
end

local function CreateButtonFromItem(item)
    -- This wrapper frame allows the calls to self:GetParent():GetID() to
    -- function properly in the inherited Blizzard UI code, since the
    -- parent ID is expected to be the bagid and the child ID is the slot.
    local basename
    local buttonTemplate 
    
    if item.bagid == -1 then
        basename = "SackvilleBankItemButton" .. item.bagslot
        buttonTemplate = "BankItemButtonGenericTemplate"
    else
        basename = "SackvilleItemButton" .. item.bagslot
        buttonTemplate = "ContainerFrameItemButtonTemplate"
    end
    
    local wrapper = _G[basename.."_wrapper"]
        or CreateFrame("Frame", basename.."_wrapper", Sackville.dummyFrame)
    wrapper:ClearAllPoints()
    wrapper:SetPoint("TOPLEFT")
    wrapper:SetID(item.bagid)
    wrapper:SetHeight(Sackville.Options.BUTTON_SIZE)
    wrapper:SetWidth(Sackville.Options.BUTTON_SIZE)
    wrapper:Show()
    -- 
    local button = _G[basename] or 
        CreateFrame("Button", basename, wrapper, buttonTemplate)
    button:SetAllPoints() -- fill the parent frame
    button:SetID(item.slot)
    SetItemButtonTexture(button, item.texture)
    SetItemButtonCount(button, item.count)
    button.wrapper = wrapper
    button:Show()
    -- Sackville:Debug(3, "main", "Created button %s, SetID(%d)", 
        -- item.button:GetName(), item.slot)
    return button
end

function Sackville.Item:UpdateQuestInfo()
    self.isQuestItem, self.questID, self.isActive = 
        GetContainerItemQuestInfo(self.bagid, self.slot)
    SetQualityOverlay(self)
end

function Sackville.Item:New(bagid, slot)
    local self = {}
    setmetatable(self, Sackville.Item)
    local bagslot = bagid .. "-" .. slot

    local itemid = GetContainerItemID(bagid, slot)
    local _, itemCount = GetContainerItemInfo(bagid, slot)
    if itemid then
        local itemName, _, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
            itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemid) 
        local isQuestItem, questID, isActive = GetContainerItemQuestInfo(bagid, slot)
        
        self.empty = false
        self.id = itemid
        self.count = itemCount
        self.name = itemName or "Unknown item"
        self.quality = itemRarity
        self.level = itemLevel
        self.minLevel = itemMinLevel
        self.equipLoc = itemEquipLoc
        self.equipSlot = equippedSlot[itemEquipLoc]
        self.type = itemType
        self.subType = itemSubType
        self.stackCount = itemStackCount
        self.texture = itemTexture
        self.bagid = bagid
        self.slot = slot
        self.bagslot = bagslot
        self.bagName = ""
        self.sectionName = ""        
        self.isQuestItem = isQuestItem
        self.questID = questID
        self.isActive = isActive
        Sackville:Debug(2, "item", "  Creating new button for %s", self.name)
    else
        self.empty = true
        self.count = 0
        self.quality = 0
        self.bagid = bagid
        self.slot = slot
        self.bagslot = bagslot
        self.bagName = ""
        self.sectionName = ""
        self.texture = nil
    end
    self.button = CreateButtonFromItem(self)
    SetQualityOverlay(self)
    UpdateItemButtonCooldown(self)
    SetItemButtonCount(self.button, self.count)
    
    Sackville:Debug(1, "item", "Item:New (%s)", self:Dump())
    return self
end

function Sackville.Item:Reload()
    local itemWasRemoved = nil -- non-empty to empty
    local itemWasAdded = nil -- empty to non-empty
    local itemWasChanged = nil -- non-empty to non-empty

    local itemid = GetContainerItemID(self.bagid, self.slot)
    local _, itemCount = GetContainerItemInfo(self.bagid, self.slot)
    local savedItem = Sackville.db.char.items[self.bagslot]
    
    if itemid then
        itemWasRemoved = false
        
        if savedItem.empty then
                itemWasAdded = true
                itemWasChanged = false
        else
            -- TODO: Implement a metatable comparison
            if savedItem.id == itemid and savedItem.count == itemCount then
                itemWasAdded = false
                itemWasChanged = false
            else
                itemWasAdded = false
                itemWasChanged = true
            end
        end
    else
        itemWasAdded = false
        itemWasChanged = false
        itemWasRemoved = not savedItem.empty
    end
    
    assert (not (itemWasAdded and itemWasChanged))
    assert (itemWasAdded ~= nil and itemWasChanged ~= nil and itemWasRemoved ~= nil)
    
    self = savedItem or {}
    if itemWasRemoved then
        self.empty = true
        self.id = 0
        self.count = 0
        self.name = "empty slot"
        self.quality = nil
        self.level = nil
        self.minLevel = nil
        self.equipLoc = nil
        self.type = nil
        self.subType = nil
        self.stackCount = nil
        self.texture = nil   
        self.isQuestItem = false
        self.questID = nil
        self.isActive = false
        self:UpdateButton()
    elseif itemWasAdded or itemWasChanged then
        local itemName, _, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
            itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemid) 
        local isQuestItem, questID, isActive = GetContainerItemQuestInfo(self.bagid, self.slot)
        
        self.empty = false
        self.id = itemid
        self.count = itemCount
        self.name = itemName or "Unknown item"
        self.quality = itemRarity
        self.level = itemLevel
        self.minLevel = itemMinLevel
        self.equipLoc = itemEquipLoc
        self.type = itemType
        self.subType = itemSubType
        self.stackCount = itemStackCount
        self.texture = itemTexture      
        self.isQuestItem = isQuestItem
        self.questID = questID
        self.isActive = isActive           
        self:UpdateButton()
    else
        -- The item was empty before and is empty now
    end
      
    Sackville:Debug(3, "item", "-- Loaded item: %s, R:%s, A:%s, C:%s", 
        self.bagslot, itemWasRemoved and "true" or "false",
        itemWasAdded and "true" or "false", itemWasChanged and "true" or "false")
    local status
    if itemWasRemoved then
        status = "removed"
    elseif itemWasAdded then
        status = "added"
    elseif itemWasChanged or (self.forceUpdate and not self.empty) then
        status = "changed"
    else
        -- Item was empty before and is empty now. Nothing to do.
    end
    
    self.forceUpdate = false
    return status
end

function Sackville.Item:FlagForUpdate()
    self.forceUpdate = true
end

function Sackville.Item:UpdateButton()
    Sackville:Debug(2, "item", "Updating button for %s with id %d at %s",
        self.name, self.id, self.bagslot)
    
    SetItemButtonTexture(self.button, self.texture)
    SetItemButtonCount(self.button, self.count)
    SetQualityOverlay(self)
end

function Sackville.Item:Show()
    self.button.wrapper:Show()
end

function Sackville.Item:Hide()
    self.button.wrapper:Hide()
end

function Sackville.Item:SetParent(parentFrame)
    self.button.wrapper:SetParent(parentFrame)
end

function Sackville.Item:SetButtonDesaturated(status)
    SetItemButtonDesaturated(self.button, status)
end

function Sackville.Item:SetBagName(bagName)
    self.bagName = bagName
end

function Sackville.Item:SetSectionName(sectionName)
    self.sectionName = sectionName
end

function Sackville.Item:SetEmpty(value)
    self.empty = value
end

function Sackville.Item:IsEmpty()
    return self.empty
end

function Sackville.Item:IsQuestItem()
    -- if self.isQuestItem or self.questID or self.type == BL["Quest"] then
    if self.questID then
        return true
    else
        return false
    end
end

function Sackville.Item:GetName()
    return self.name
end

function Sackville.Item:GetBagSlot()
    return self.bagslot
end

function Sackville.Item:SetPoint(...)
    self.button.wrapper:ClearAllPoints()
    self.button.wrapper:SetPoint(...)
end

function Sackville.Item:GetBagId()
    return self.bagid
end

function Sackville.Item:SetButtonCount(count)
    SetItemButtonCount(self.button, count)
end

function Sackville.Item:UpdateButtonCooldown()
    UpdateItemButtonCooldown(self)
end

function Sackville.Item:Reinitialize()
    self:RecreateButton()
    self:ReAddToSection()
end
function Sackville.Item:RecreateButton()
    self.button = CreateButtonFromItem(self)
    SetQualityOverlay(self)
    UpdateItemButtonCooldown(self)
    SetItemButtonCount(self.button, self.count)
end

function Sackville.Item:ReAddToSection()
    Sackville:Debug(1, "item", "Item:ReAdd (%s)", self:Dump())
    if self.bagName ~= "" and self.sectionName ~= "" then
        local bag = Sackville:GetBag(self.bagName)
        local section = bag.sections[self.sectionName]
        section:AddItem(self)
    end
end

-- Remove self from whatever section we are in
function Sackville.Item:Release()
    Sackville:Debug(1, "item", "Releasing (%s)", self:Dump())
    local bag = Sackville:GetBag(self.bagName)
    local section = bag.sections[self.sectionName]
    section:RemoveItem(self)
    -- self.empty = true
end

function Sackville.Item:GetBagType()
    if Sackville:IsPlayerBagId(self.bagid) then
        return PLAYER
    elseif Sackville:IsBankBagId(self.bagid) then
        return L["Bank"]
    end
end

function Sackville.Item:Dump()
    local id = self.id or "nil"
    local name = self.name or "nil"
    local bagslot = self.bagslot or "nil"
    local empty = self.empty and "Yes" or "no"
    -- return sprintf("id: %s, name: %s, bagslot: %s, type: %s, subType: %s, empty: %s",
        -- self.id, self.name, self.bagslot, self.type, self.subType, empty)
    local bagName = self.bagName or "nil"
    local sectionName = self.sectionName or "nil"
    local itemType = self.type or "nil"
    local subType = self.subType or "nil"
    return sprintf("id: %d, name: %s, bagslot: %s, B:%s/S:%s, empty: %s, type:%s, subType:%s",
        id, name, bagslot, bagName, sectionName, empty, itemType, subType)
end