local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville")
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")
local B = LibStub("LibBabble-Inventory-3.0")
local BL = B:GetLookupTable()


Sackville.Options = {}

local sep = "\001"
local bank_prefix = "(" .. L["Bank"] .. ") "

Sackville.Options.categories = {
    [BL["Armor"]] = {
        {   filterPath = BL["Armor"] }
    },
    [BL["Consumable"]] = {
        {   filterPath = BL["Consumable"] }
    },
    [BL["Weapon"]] = {
        {   filterPath = BL["Weapon"] }
    },
    [BL["Trinket"]] = {
        {   filterPath = L["Equipment"] .. sep .. BL["Trinket"] }
    },
    [BL["Tabard"]] = {
        {   filterPath = L["Equipment"] .. sep .. BL["Tabard"] }
    },
    [BL["Food & Drink"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Food & Drink"] }
    },
    [BL["Potion"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Potion"] }
    },
    [BL["Elixir"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Elixir"] }
    },
    [BL["Flask"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Flask"] }
    },
    [BL["Scroll"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Scroll"] }
    },
    [BL["Bandage"]] = {
        {   filterPath = BL["Consumable"] .. sep .. BL["Bandage"] }
    },
    [BL["Metal & Stone"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Metal & Stone"] }
    },
    [BL["Jewelcrafting"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Jewelcrafting"],
            operation = "OR"
        },
        {   filterPath = BL["Gem"],
            operation = "OR"
        }
    },
    [BL["Enchanting"]] = {
        {   
            filterPath = BL["Trade Goods"] .. sep .. BL["Enchanting"],
            operation = "OR"
        },
        {
            filterPath = BL["Trade Goods"] .. sep .. BL["Weapon Enchantment"],
            operation = "OR"
        },
        {
            filterPath = BL["Trade Goods"] .. sep .. BL["Armor Enchantment"],
            operation = "OR"
        }
    },
    [BL["Trade Goods"]] = {
        {   filterPath = BL["Trade Goods"] }
    },
    [BL["Cloth"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Cloth"] }
    },
    [BL["Parts"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Parts"] }
    },
    [BL["Elemental"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Elemental"] }
    },
    [BL["Herb"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Herb"] }
    },
    [BL["Leather"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Leather"] }
    },
    [BL["Devices"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Devices"] }
    },
    [BL["Cooking"]] = {
        {   filterPath = BL["Trade Goods"] .. sep .. BL["Cooking"] }
    },
    [BL["Quest"]] = {
        {   filterPath = BL["Quest"] .. sep .. BL["Quest"] }
    },
    [BL["Reagent"]] = {
        {   filterPath = BL["Miscellaneous"] .. sep .. BL["Reagent"] }
    },
    [BL["Junk"]] = {
        {   filterPath = QUALITY .. sep .. ITEM_QUALITY0_DESC }
    },
    [BL["Recipe"]] = {
        {   filterPath = BL["Recipe"] }
    }
}

Sackville.Options.bagConfig = {
    [L["Equipment"]] = {
        name = L["Equipment"],
        displayOrder = 1,
        bagType = PLAYER,
        defaultPriority = 2,
        defaultSortCriteria = { 
            {
                key = "level",
                comparison = ">"
            },
            {
                key = "quality",
                comparison = ">"
            },
            {
                key = "name",
                comparison = "<"
            }        
        },
        sections = {
            [BL["Armor"]] = {
                name = BL["Armor"],
                displayOrder = 1,
                priority = 1,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Armor"]
                }
            },
            [BL["Weapon"]] = {
                name = BL["Weapon"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Weapon"]
                }
            },
            [BL["Trinket"]] = {
                name = BL["Trinket"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Trinket"]
                }
            },
            [BL["Tabard"]] = {
                name = BL["Tabard"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Tabard"]
                }
            }
        }
    },
    [BL["Consumable"]] = {
        name = BL["Consumable"],
        bagType = PLAYER,
        defaultPriority = 2,
        displayOrder = 2,
        defaultSortCriteria = { 
            {
                key = "minLevel",
                comparison = ">"
            },
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Food & Drink"]] = {
                name = BL["Food & Drink"],
                displayOrder = 1,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Food & Drink"]
                }
            },
            [BL["Flask"]] = {
                name = BL["Flask"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Flask"]
                }
            },
            [BL["Potion"]] = {
                name = BL["Potion"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Potion"]
                }
            },
            [BL["Elixir"]] = {
                name = BL["Elixir"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elixir"]
                }
            },
            [BL["Scroll"]] = {
                name = BL["Scroll"],
                displayOrder = 5,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Scroll"]
                }
            },
            [BL["Bandage"]] = {
                name = BL["Bandage"],
                displayOrder = 6,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Bandage"]
                }
            },
            [BL["Other"]] = {
                name = BL["Other"],
                displayOrder = 7,
                priority = 1,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Consumable"]
                }
            }             
        }
    },
    [BL["Trade Goods"]] = {
        name = BL["Trade Goods"],
        displayOrder = 3,
        bagType = PLAYER,
        defaultPriority = 2,
        defaultSortCriteria = { 
            {
                key = "quality",
                comparison = ">"
            },        
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Metal & Stone"]] = {
                name = BL["Metal & Stone"],
                displayOrder = 1,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Metal & Stone"]
                }
            },
            [BL["Jewelcrafting"]] = {
                name = BL["Jewelcrafting"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = {
                    BL["Jewelcrafting"]
                }
            },
            [BL["Enchanting"]] = {
                name = BL["Enchanting"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Enchanting"]
                }
            },
            [BL["Cloth"]] = {
                name = BL["Cloth"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Cloth"]
                }
            },
            [BL["Parts"]] = {
                name = BL["Parts"],
                displayOrder = 5,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Parts"]
                }
            },
            [BL["Elemental"]] = {
                name = BL["Elemental"],
                displayOrder = 6,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elemental"]
                }
            },
            [BL["Herb"]] = {
                name = BL["Herb"],
                displayOrder = 7,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Herb"]
                }
            },
            [BL["Leather"]] = {
                name = BL["Leather"],
                displayOrder = 8,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Leather"]
                }
            },
            [BL["Cooking"]] = {
                name = BL["Cooking"],
                displayOrder = 9,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Cooking"]
                }
            },
            [BL["Recipe"]] = {
                name = BL["Recipe"],
                displayOrder = 10,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Recipe"]
                }
            },        
            [BL["Other"]] = {
                name = BL["Other"],
                displayOrder = 99,
                priority = 1,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elemental"]
                }
            }         
        }
    },
    [BL["Other"]] = {
        name = BL["Other"],
        displayOrder = 4,
        bagType = PLAYER,
        defaultPriority = 1,
        defaultSortCriteria = { 
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Quest"]] = {
                name = BL["Quest"],
                displayOrder = 1,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Quest"]
                }
            },
            [BL["Reagent"]] = {
                name = BL["Reagent"],
                displayOrder = 2,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Reagent"]
                }
            },
            [BL["Devices"]] = {
                name = BL["Devices"],
                displayOrder = 3,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Devices"]
                }
            },
            [BL["Junk"]] = {
                name = BL["Junk"],
                displayOrder = 4,
                priority = 10,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Junk"]
                }
            },
            [BL["Other"]] = {
                name = BL["Other"],
                displayOrder = 5,
                priority = 0,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                alwaysMatch = true
            },
            [EMPTY] = {
                name = EMPTY,
                displayOrder = 6,
                priority = 0,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                showEmptySlot = true,
                alwaysShow = true
            }            
        }
    },
    [bank_prefix .. L["Equipment"]] = {
        name = bank_prefix .. L["Equipment"],
        bagType = L["Bank"],
        defaultPriority = 2,
        displayOrder = 5,
        defaultSortCriteria = { 
            {
                key = "level",
                comparison = ">"
            },
            {
                key = "quality",
                comparison = ">"
            },
            {
                key = "name",
                comparison = "<"
            }        
        },
        sections = {
            [BL["Armor"]] = {
                name = BL["Armor"],
                displayOrder = 1,
                priority = 1,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Armor"]
                }
            },
            [BL["Weapon"]] = {
                name = BL["Weapon"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Weapon"]
                }
            },
            [BL["Trinket"]] = {
                name = BL["Trinket"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Trinket"]
                }
            },
            [BL["Tabard"]] = {
                name = BL["Tabard"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Tabard"]
                }
            }
        }
    },
    [bank_prefix .. BL["Consumable"]] = {
        name = bank_prefix .. BL["Consumable"],
        bagType = L["Bank"],
        defaultPriority = 2,
        displayOrder = 6,
        defaultSortCriteria = { 
            {
                key = "minLevel",
                comparison = ">"
            },
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Food & Drink"]] = {
                name = BL["Food & Drink"],
                displayOrder = 1,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Food & Drink"]
                }
            },
            [BL["Flask"]] = {
                name = BL["Flask"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Flask"]
                }
            },
            [BL["Potion"]] = {
                name = BL["Potion"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Potion"]
                }
            },
            [BL["Elixir"]] = {
                name = BL["Elixir"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elixir"]
                }
            },
            [BL["Scroll"]] = {
                name = BL["Scroll"],
                displayOrder = 5,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Scroll"]
                }
            },
            [BL["Bandage"]] = {
                name = BL["Bandage"],
                displayOrder = 6,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Bandage"]
                }
            },
            [BL["Other"]] = {
                name = BL["Other"],
                priority = 1,
                displayOrder = 6,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Consumable"]
                }
            },              
        }
    },
    [bank_prefix .. BL["Trade Goods"]] = {
        name = bank_prefix .. BL["Trade Goods"],
        bagType = L["Bank"],
        defaultPriority = 2,
        displayOrder = 7,
        defaultSortCriteria = { 
            {
                key = "quality",
                comparison = ">"
            },
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Metal & Stone"]] = {
                name = BL["Metal & Stone"],
                displayOrder = 1,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Metal & Stone"]
                }
            },
            [BL["Jewelcrafting"]] = {
                name = BL["Jewelcrafting"],
                displayOrder = 2,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = {
                    BL["Jewelcrafting"]
                }
            },
            [BL["Enchanting"]] = {
                name = BL["Enchanting"],
                displayOrder = 3,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Enchanting"]
                }
            },
            [BL["Cloth"]] = {
                name = BL["Cloth"],
                displayOrder = 4,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Cloth"]
                }
            },
            [BL["Parts"]] = {
                name = BL["Parts"],
                displayOrder = 5,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Parts"]
                }
            },
            [BL["Elemental"]] = {
                name = BL["Elemental"],
                displayOrder = 6,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elemental"]
                }
            },
            [BL["Herb"]] = {
                name = BL["Herb"],
                displayOrder = 7,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Herb"]
                }
            },
            [BL["Leather"]] = {
                name = BL["Leather"],
                displayOrder = 8,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Leather"]
                }
            },
            [BL["Cooking"]] = {
                name = BL["Cooking"],
                displayOrder = 9,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Cooking"]
                }
            },
            [BL["Recipe"]] = {
                name = BL["Recipe"],
                displayOrder = 10,
                priority = 2,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Recipe"]
                }
            },
            [BL["Other"]] = {
                name = BL["Other"],
                displayOrder = 99,
                priority = 1,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Elemental"]
                }
            }         
        }
    },
    [bank_prefix .. BL["Other"]] = {
        name = bank_prefix .. BL["Other"],
        bagType = L["Bank"],
        defaultPriority = 1,
        displayOrder = 8,
        defaultSortCriteria = { 
            {
                key = "name",
                comparison = "<"
            },
            {
                key = "count",
                comparison = ">"
            }        
        },
        sections = {
            [BL["Quest"]] = {
                name = BL["Quest"],
                displayOrder = 1,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Quest"]
                }
            },
            [BL["Reagent"]] = {
                name = BL["Reagent"],
                displayOrder = 2,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Reagent"]
                }
            },
            [BL["Devices"]] = {
                name = BL["Devices"],
                displayOrder = 3,
                priority = 1,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Devices"]
                }
            },
            [BL["Junk"]] = {
                name = BL["Junk"],
                displayOrder = 4,
                priority = 10,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                categories = { 
                    BL["Junk"]
                }
            },
            [BL["Other"]] = {
                name = BL["Other"],
                displayOrder = 5,
                priority = 0,
                useDefaultPriority = false,
                useDefaultSorting = true,
                sortCriteria = {},
                alwaysMatch = true
            },
            [EMPTY] = {
                name = EMPTY,
                displayOrder = 6,
                priority = 0,
                useDefaultPriority = true,
                useDefaultSorting = true,
                sortCriteria = {},
                showEmptySlot = true,
                alwaysShow = true
            }
        }
    }

}

Sackville.Options.layout = {}
Sackville.Options.layout.origin = {
    anchor = "BOTTOMRIGHT",
    x_offset = -3,
    y_offset = 90
}

Sackville.Options.BUTTON_SIZE = 40
Sackville.Options.FRAME_PADDING = 0
Sackville.Options.TITLE_EXTRA_WIDTH = 80

Sackville.Options.bagTypeList = {
    [PLAYER] = PLAYER,
    [L["Bank"]] = L["Bank"]
}

Sackville.Options.itemSortKeyList = {
    -- Dropdown name = item key
    ["count"]   = L["Count"],
    ["name"]    = NAME,
    ["quality"] = QUALITY,
    ["level"]   = STAT_AVERAGE_ITEM_LEVEL,
    ["minLevel"]= L["Required level"],
    ["equipSlot"]=L["Equipped slot"],
}

Sackville.Options.itemSortOperatorList = {
    -- Dropdown name = item key
    ["<"] = L["Ascending"],
    [">"] = L["Descending"],
}

Sackville.Options.bag = {
    margin_left = 15,
    margin_right = 15,
    margin_top = 30,
    margin_bottom = 15,
    padding_left = 0,
    padding_right = 0,
    padding_top = 0,
    padding_bottom = 0,
    max_width = 450,
    min_width = 40,
    min_height = 135,
    title_height = 32,
    strata = "HIGH"
}

Sackville.Options.bag.backdrop = {
  -- path to the background texture
  bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark",  
  -- path to the border texture
  edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
  -- true to repeat the background texture to fill the frame, false to scale it
  tile = true,
  -- size (width or height) of the square repeating background tiles (in pixels)
  tileSize = 32,
  -- thickness of edge segments and square size of edge corners (in pixels)
  edgeSize = 22,
  -- distance from the edges of the frame to those of the background texture (in pixels)
  insets = {
    left = 6,
    right = 6,
    top = 6,
    bottom = 6
  }
}

Sackville.Options.bag.backdrop_bank = {
  -- path to the background texture
  -- bgFile = "Interface/Addons/Sackville/Textures/UI-Blue-Background",
  bgFile = 'Interface\\Addons\\Sackville\\Textures\\UI-Blue-Background.blp',
  -- path to the border texture
  edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
  -- true to repeat the background texture to fill the frame, false to scale it
  tile = true,
  -- size (width or height) of the square repeating background tiles (in pixels)
  tileSize = 32,
  -- thickness of edge segments and square size of edge corners (in pixels)
  edgeSize = 22,
  -- distance from the edges of the frame to those of the background texture (in pixels)
  insets = {
    left = 6,
    right = 6,
    top = 6,
    bottom = 6
  }
}

Sackville.Options.bag.backdrop_title = {
   bgFile = "Interface/WorldStateFrame/WorldStateFinalScoreFrame-TopBackground",  
   edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
   -- true to repeat the background texture to fill the frame, false to scale it
   tile = false,
   -- size (width or height) of the square repeating background tiles (in pixels)
   tileSize = 16,
   -- thickness of edge segments and square size of edge corners (in pixels)
   edgeSize = 22,
   -- distance from the edges of the frame to those of the background texture (in pixels)
   insets = {
      left = 6,
      right = 6,
      top = 6,
      bottom = 0
   }
}

Sackville.Options.section = {
    padding_left = 0,   -- unused
    padding_right = 10,
    padding_top = 0,    -- unused
    padding_bottom = 3,
    margin_left = 0,    -- unused
    margin_right = 0,   -- unused
    margin_top = 15,
    margin_bottom = 0,  -- unused
    min_height = Sackville.Options.BUTTON_SIZE + 20,
    title_height = 13
}

Sackville.Options.defaults = {
    profile = {
        bagConfig = Sackville.Options.bagConfig,
        categories = Sackville.Options.categories,
    },
    char = {
        items = {},
        emptyItem = {},
        bagState = {},
        prevEmptyBagName = {}
    }
}